/**
 * @file
 * The main code of the zh_rf24 component.
 */

#include "zh_rf24.h"

/// \cond
#define ZH_RF24_DATA_SEND_SUCCESS BIT0
#define ZH_RF24_DATA_SEND_FAIL BIT1
#define ZH_RF24_MAX_MESSAGE_SIZE 32
#define ZH_SPI_HOST SPI2_HOST
/// \endcond

static void s_zh_rf24_processing(void *pvParameter);
static void s_zh_rf24_irq_isr_handler(void *arg);
static void s_zh_rf24_irq_processing_task(void *pvParameter);

static const char *TAG = "zh_rf24";

static EventGroupHandle_t s_zh_rf24_send_cb_status_event_group_handle = {0};
static QueueHandle_t s_zh_rf24_queue_handle = {0};
static TaskHandle_t s_zh_rf24_processing_task_handle = {0};
static SemaphoreHandle_t s_zh_rf24_irq_semaphore = {0};
static zh_rf24_init_config_t s_zh_rf24_init_config = {0};
static spi_device_handle_t s_zh_rf24_spi_handle = {0};
static bool s_zh_rf24_is_initialized = false;

/// \cond
typedef enum
{
    ZH_RF24_RECV,
    ZH_RF24_SEND,
} __attribute__((packed)) zh_rf24_queue_id_t;

typedef struct
{
    uint8_t pipe;
    bool confirm;
    uint8_t *data;
    uint8_t data_len;
} __attribute__((packed)) zh_rf24_queue_data_t;

typedef struct
{
    zh_rf24_queue_id_t id;
    zh_rf24_queue_data_t data;
} __attribute__((packed)) zh_rf24_queue_t;

ESP_EVENT_DEFINE_BASE(ZH_RF24);
/// \endcond

esp_err_t zh_rf24_init(zh_rf24_init_config_t *config)
{
    ESP_LOGI(TAG, "RF24 initialization begin.");
    if (config == NULL)
    {
        ESP_LOGE(TAG, "RF24 initialization fail. Invalid argument.");
        return ESP_ERR_INVALID_ARG;
    }
    s_zh_rf24_init_config = *config;
    s_zh_rf24_send_cb_status_event_group_handle = xEventGroupCreate();
    s_zh_rf24_queue_handle = xQueueCreate(s_zh_rf24_init_config.queue_size, sizeof(zh_rf24_queue_t));
    gpio_config_t pin_config = {0};
    pin_config.intr_type = GPIO_INTR_DISABLE;
    pin_config.mode = GPIO_MODE_OUTPUT;
    pin_config.pin_bit_mask = (1ULL << s_zh_rf24_init_config.ce_pin);
    pin_config.pull_down_en = GPIO_PULLDOWN_DISABLE;
    pin_config.pull_up_en = GPIO_PULLUP_DISABLE;
    if (gpio_config(&pin_config) != ESP_OK)
    {
        ESP_LOGE(TAG, "RF24 initialization fail. Incorrect GPIO number.");
        return ESP_ERR_INVALID_ARG;
    }
    gpio_set_level(s_zh_rf24_init_config.ce_pin, 0);
    pin_config.intr_type = GPIO_INTR_DISABLE;
    pin_config.mode = GPIO_MODE_OUTPUT;
    pin_config.pin_bit_mask = (1ULL << s_zh_rf24_init_config.csn_pin);
    pin_config.pull_down_en = GPIO_PULLDOWN_DISABLE;
    pin_config.pull_up_en = GPIO_PULLUP_DISABLE;
    if (gpio_config(&pin_config) != ESP_OK)
    {
        ESP_LOGE(TAG, "RF24 initialization fail. Incorrect GPIO number.");
        return ESP_ERR_INVALID_ARG;
    }
    gpio_set_level(s_zh_rf24_init_config.csn_pin, 1);
    pin_config.intr_type = GPIO_INTR_NEGEDGE;
    pin_config.mode = GPIO_MODE_INPUT;
    pin_config.pin_bit_mask = (1ULL << s_zh_rf24_init_config.irq_pin);
    pin_config.pull_down_en = GPIO_PULLDOWN_DISABLE;
    pin_config.pull_up_en = GPIO_PULLUP_ENABLE;
    if (gpio_config(&pin_config) != ESP_OK)
    {
        ESP_LOGE(TAG, "RF24 initialization fail. Incorrect GPIO number.");
        return ESP_ERR_INVALID_ARG;
    }
    spi_bus_config_t spi_bus_config = {
        .sclk_io_num = s_zh_rf24_init_config.sck_pin,
        .mosi_io_num = s_zh_rf24_init_config.mosi_pin,
        .miso_io_num = s_zh_rf24_init_config.miso_pin,
        .quadwp_io_num = -1,
        .quadhd_io_num = -1};
    if (spi_bus_initialize(ZH_SPI_HOST, &spi_bus_config, SPI_DMA_CH_AUTO) != ESP_OK)
    {
        ESP_LOGE(TAG, "RF24 initialization fail. SPI initialization error.");
        return ESP_FAIL;
    }
    spi_device_interface_config_t spi_device_interface_config = {
        .clock_speed_hz = 4000000,
        .spics_io_num = -1,
        .queue_size = 7,
        .mode = 0,
        .flags = SPI_DEVICE_NO_DUMMY};
    if (spi_bus_add_device(ZH_SPI_HOST, &spi_device_interface_config, &s_zh_rf24_spi_handle) != ESP_OK)
    {
        ESP_LOGE(TAG, "RF24 initialization fail. SPI initialization error.");
        return ESP_FAIL;
    }
    s_zh_rf24_irq_semaphore = xSemaphoreCreateBinary();
    if (gpio_install_isr_service(0) != ESP_OK || gpio_isr_handler_add(s_zh_rf24_init_config.irq_pin, s_zh_rf24_irq_isr_handler, NULL) != ESP_OK)
    {
        ESP_LOGE(TAG, "RF24 initialization fail. Internal error.");
        return ESP_FAIL;
    }
    if (xTaskCreatePinnedToCore(&s_zh_rf24_irq_processing_task, "NULL", s_zh_rf24_init_config.stack_size, NULL, s_zh_rf24_init_config.task_priority, NULL, tskNO_AFFINITY) != pdPASS)
    {
        ESP_LOGE(TAG, "RF24 initialization fail. Internal error.");
        return ESP_FAIL;
    }
    s_zh_rf24_is_initialized = true;
    ESP_LOGI(TAG, "RF24 initialization success.");
    return ESP_OK;
}

esp_err_t zh_rf24_deinit(void)
{
    return 0;
}

esp_err_t zh_rf24_send(const uint8_t *data, const uint8_t data_len, const bool confirm)
{
    return 0;
}

static void s_zh_rf24_processing(void *pvParameter)
{
}

static void s_zh_rf24_irq_isr_handler(void *arg)
{
    xSemaphoreGiveFromISR(s_zh_rf24_irq_semaphore, NULL);
}

static void s_zh_rf24_irq_processing_task(void *pvParameter)
{
    for (;;)
    {
        xSemaphoreTake(s_zh_rf24_irq_semaphore, portMAX_DELAY);
        // switch_config->gpio_processing = true;
        // switch_config->status.status = (switch_config->status.status == HAONOFT_ON) ? HAONOFT_OFF : HAONOFT_ON;
        // zh_gpio_set_level(switch_config);
        // zh_save_status(switch_config);
        // if (switch_config->gateway_is_available == true)
        // {
        //     zh_send_switch_status_message(switch_config);
        // }
        // vTaskDelay(500 / portTICK_PERIOD_MS); // To prevent button contact rattling. Value is selected experimentally.
        // switch_config->gpio_processing = false;
    }
    vTaskDelete(NULL);
}